/**
 * Copyright (C) 2010 Cloudfarming <info@cloudfarming.nl>
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package nl.cloudfarming.client.geoviewer.jxmap.panels;

import com.vividsolutions.jts.geom.Point;
import java.awt.EventQueue;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.Collections;
import java.util.HashMap;
import java.util.Map;
import java.util.Set;
import javax.swing.JPanel;
import nl.cloudfarming.client.geoviewer.Category;
import nl.cloudfarming.client.geoviewer.Layer;
import nl.cloudfarming.client.geoviewer.LayerEventKey;
import nl.cloudfarming.client.geoviewer.LayerInfo;
import nl.cloudfarming.client.geoviewer.LayerObject;
import nl.cloudfarming.client.geoviewer.LayerObjectSelection;
import nl.cloudfarming.client.geoviewer.MapController;
import nl.cloudfarming.client.geoviewer.Preset;
import nl.cloudfarming.client.geoviewer.layers.LayerDataManager;
import nl.cloudfarming.eventbus.GuiEvent;
import nl.cloudfarming.eventbus.GuiEventBus;
import nl.cloudfarming.eventbus.GuiEventKey;
import nl.cloudfarming.eventbus.GuiEventListener;
import org.jdesktop.swingx.JXMapViewer;
import org.jdesktop.swingx.mapviewer.GeoPosition;

/**
 * Controller of the rootmap panel.
 * 
 *
 * @author Timon Veenstra
 */
public class RootMapPanelController implements MapController {

    private final LayerDataManager layerDataManager;
    //TODO should root map panel get an explorer manager?
    private final RootMapPanel rootMapPanel;
    private final Map<Layer<LayerObject<?, ?>>, LayerPanel> layerPanelMap = new HashMap<Layer<LayerObject<?, ?>>, LayerPanel>();
    private boolean active = false;

    public RootMapPanelController() {
        layerDataManager = LayerDataManager.getInstance();
        rootMapPanel = new RootMapPanel(layerDataManager.getExplorerManager());
        //
        // initialize the layers
        //
        initLayers();
        //
        // register property change listener to handle change in node selection.
        //
        SelectionChangeListener changeListener = new SelectionChangeListener();
        GuiEventBus.addListener(changeListener);

        NewLayerListener layerListener = new NewLayerListener();
        GuiEventBus.addListener(layerListener);

        MapController.Registry.addMapController(this);

    }

    private void initLayers() {
        for (Layer layer: layerDataManager.getLayers()){
            addLayer(layer);
        }
    }

    public JPanel getPanel() {
        return this.rootMapPanel;
    }

    private void repaintLayer(final Layer layer) {
        if (EventQueue.isDispatchThread()) {
            layerPanelMap.get(layer).getView().repaint();
        } else {
            EventQueue.invokeLater(new Runnable() {

                @Override
                public void run() {
                    layerPanelMap.get(layer).getView().repaint();
                }
            });
        }
    }

    private void addLayer(final Layer layer) {
        //
        // register listener for pallete change
        //
        layer.addPropertyChangeListener(new PropertyChangeListener() {

            @Override
            public void propertyChange(PropertyChangeEvent evt) {
                if (Layer.PROPERTY_PALETTE.equals(evt.getPropertyName())) {
                    repaintLayer(layer);
                }
                if (Layer.PROPERTY_OBJECTS.equals(evt.getPropertyName())) {
                    repaintLayer(layer);
                }
            }
        });

        LayerPanel layerPanel = new LayerPanel(this, rootMapPanel, layer);

        layerPanelMap.put(layer, layerPanel);
        rootMapPanel.addLayerPanel(layerPanel);
    }

    @Override
    public Set<Layer<LayerObject<?, ?>>> getLayers() {
        return Collections.unmodifiableSet(layerPanelMap.keySet());
    }

    @Override
    public void setLayerOrder(int[] order, Category category) {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    @Override
    public LayerInfo getLayerInfo(Layer layer) {
        assert layerPanelMap.containsKey(layer);
        return layerPanelMap.get(layer);
    }

    @Override
    public Preset getCurrentPreset() {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    @Override
    public String getMapName() {
        return "map:" + System.identityHashCode(this);
    }

    JXMapViewer getViewer() {
        return rootMapPanel.getMapViewer();
    }

    @Override
    public void zoomToObject(LayerObject layerObject, Layer layer) {
        assert this.layerPanelMap.containsKey(layer);

        Point centroid = layerObject.getGeometry().getCentroid();
        GeoPosition geoPosition = new GeoPosition(centroid.getY(), centroid.getX());
        rootMapPanel.getMapViewer().setCenterPosition(geoPosition);
        rootMapPanel.getMapViewer().setZoom(3);
    }

    public boolean isActive() {
        return active;
    }

    public void setActive(boolean active) {
        this.active = active;
    }

    private class NewLayerListener implements GuiEventListener<Layer> {

        @Override
        public void onEvent(GuiEvent<Layer> event) {
            addLayer(event.getContent());
        }

        @Override
        public boolean listensTo(GuiEventKey guiEventKey) {
            return LayerEventKey.NEW_LAYER.equals(guiEventKey);
        }

        @Override
        public String getModuleName() {
            return "geoviewer-jxmap";
        }
    }

    private class SelectionChangeListener implements GuiEventListener<LayerObjectSelection> {

        @Override
        public void onEvent(GuiEvent<LayerObjectSelection> event) {
            layerPanelMap.get(event.getContent().getLayer()).setSelectedObjects(event.getContent().getObjects());
        }

        @Override
        public boolean listensTo(GuiEventKey guiEventKey) {
            return LayerEventKey.SELECTION_CHANGED.equals(guiEventKey);
        }

        @Override
        public String getModuleName() {
            return "geoviewer-jxmap";
        }
    }
}
