/**
 * Copyright (C) 2010 Cloudfarming <info@cloudfarming.nl>
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package nl.cloudfarming.client.geoviewer.layers;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.beans.PropertyVetoException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.EnumMap;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import javax.swing.ActionMap;
import javax.swing.text.DefaultEditorKit;
import nl.cloudfarming.client.geoviewer.Category;
import nl.cloudfarming.client.geoviewer.Layer;
import nl.cloudfarming.client.geoviewer.LayerEventKey;
import nl.cloudfarming.client.geoviewer.LayerObject;
import nl.cloudfarming.client.geoviewer.LayerObjectSelection;
import nl.cloudfarming.client.geoviewer.LayerProvider;
import nl.cloudfarming.client.geoviewer.MapController;
import nl.cloudfarming.client.geoviewer.NewObjectSelectionEvent;
import nl.cloudfarming.client.geoviewer.jxmap.MapViewerTopComponent;
import nl.cloudfarming.eventbus.BaseGuiEventProducer;
import nl.cloudfarming.eventbus.GuiEvent;
import nl.cloudfarming.eventbus.GuiEventBus;
import nl.cloudfarming.eventbus.GuiEventKey;
import nl.cloudfarming.eventbus.GuiEventListener;
import org.openide.explorer.ExplorerManager;
import org.openide.explorer.ExplorerUtils;
import org.openide.nodes.AbstractNode;
import org.openide.nodes.Node;
import org.openide.util.Exceptions;
import org.openide.util.Lookup;
import org.openide.util.Lookup.Result;
import org.openide.util.NbBundle;

/**
 * singleton manages all layer data
 *
 *
 * @author Timon Veenstra
 */
public class LayerDataManager implements ExplorerManager.Provider, Lookup.Provider {

    // the instance
    private static LayerDataManager instance;
    // the event producer
    private final EventProducer eventProducer = new EventProducer();
    private final Result<LayerProvider> providerResult = Lookup.getDefault().lookupResult(LayerProvider.class);
    private static final String ROOT_NODE_NAME = "root.node.name";
    private ExplorerManager manager;
    private Lookup lookup;
    private final Set<Layer> layers = new HashSet<Layer>();
    private final Map<Category, Set<Layer>> categoryLayers = new EnumMap(Category.class);

    public static LayerDataManager getInstance() {
        if (instance == null) {
            instance = new LayerDataManager();
        }
        return instance;
    }

    private LayerDataManager() {
        //
        // initialize layer data
        //
        initLayers();
        //
        // initialize the layer list explorer manager
        //
        initLayerExplorerManager();
        //
        // register listener for new layers
        //
        GuiEventBus.addListener(new NewLayerListener());
    }

    /**
     * initializes the layer list explorer manager
     */
    private void initLayerExplorerManager() {
        //TODO find solution for multiple map support
        manager = new ExplorerManager();
        manager.setRootContext(new AbstractNode(new CategoryChildren()));
        manager.getRootContext().setDisplayName(NbBundle.getMessage(this.getClass(), ROOT_NODE_NAME));

        ExplorerChangeListener changeListener = new ExplorerChangeListener();
        manager.addPropertyChangeListener(changeListener);

        ActionMap map = new ActionMap();
        map.put(DefaultEditorKit.copyAction, ExplorerUtils.actionCopy(manager));
        map.put(DefaultEditorKit.cutAction, ExplorerUtils.actionCut(manager));
        map.put(DefaultEditorKit.pasteAction, ExplorerUtils.actionPaste(manager));
        map.put("delete", ExplorerUtils.actionDelete(manager, true)); // or false
        // ...but add e.g.:        InputMap keys = getInputMap(JComponent.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT);
//        keys.put(KeyStroke.getKeyStroke("control C"), DefaultEditorKit.copyAction);
//        keys.put(KeyStroke.getKeyStroke("control X"), DefaultEditorKit.cutAction);
//        keys.put(KeyStroke.getKeyStroke("control V"), DefaultEditorKit.pasteAction);
//        keys.put(KeyStroke.getKeyStroke("DELETE"), "delete");

        // ...and initialization of lookup variable
        lookup = ExplorerUtils.createLookup(manager, map);
    }

    /**
     * initialized the layer data
     */
    private void initLayers() {
        for (LayerProvider provider : providerResult.allInstances()) {
            Result<Layer> layerResult = provider.getLookup().lookupResult(Layer.class);
            for (Layer layer : layerResult.allInstances()) {
                if (!layers.contains(layer)) {
                    layers.add(layer);
                }
                addLayerToCategory(layer);
            }
        }
    }

    /**
     * add a layer to to the category map
     *
     * @param layer
     */
    private void addLayerToCategory(Layer layer) {
        Category category = layer.getCategory();
        if (categoryLayers.containsKey(category) && categoryLayers.get(category) != null) {
            categoryLayers.get(category).add(layer);
        } else {
            Set<Layer> layerSet = new HashSet<Layer>();
            layerSet.add(layer);
            categoryLayers.put(category, layerSet);
        }
    }

    /**
     * return the default explorer manager (layer)
     * @return
     */
    @Override
    public ExplorerManager getExplorerManager() {
        return manager;
    }

    /**
     * get the lookup associated with the layer explorer manager
     * @return
     */
    @Override
    public Lookup getLookup() {
        return lookup;
    }

    /**
     *
     *  to add a layer use {@link LayerDataManager#addLayer(nl.cloudfarming.client.geoviewer.Layer) }
     *  to remove  a layer use {@link LayerDataManager#removeLayer(nl.cloudfarming.client.geoviewer.Layer) }
     *
     * @return unmodifiableSet layers
     */
    public Set<Layer> getLayers() {
        return Collections.unmodifiableSet(layers);
    }

    public void removeLayer(Layer layer) {
        layers.remove(layer);
        //FIXME trigger remove layer event
        // refresh the explorer node
        LayerChildren layerChildren = NodeUtils.findLayerChildrenForCategory(layer.getCategory());
        layerChildren.addNotify();
    }

    public Set<Layer> getLayersForCategory(Category category) {
        return categoryLayers.get(category);
    }

    public void addLayer(Layer layer) {
        layers.add(layer);
        addLayerToCategory(layer);
//        eventProducer.triggerEvent(new NewLayerEvent(layer));
        // refresh the explorer node
        LayerChildren layerChildren = NodeUtils.findLayerChildrenForCategory(layer.getCategory());
        layerChildren.addNotify();
    }

    public void addObjectToSelection(LayerObject selectedObject) {
        try {
            List<Node> selectedNodes = new ArrayList<Node>(Arrays.asList(getExplorerManager().getSelectedNodes()));
            LayerObjectNode layerObjectNode = NodeUtils.findNodeForObject(selectedObject);
            if (layerObjectNode != null && !selectedNodes.contains(layerObjectNode)) {
                selectedNodes.add(layerObjectNode);
            }
            getExplorerManager().setSelectedNodes(selectedNodes.toArray(new Node[]{}));
        } catch (PropertyVetoException ex) {
            Exceptions.printStackTrace(ex);
        }
    }

    public void removeObjectFromSelection(LayerObject selectedObject) {
        try {
            List<Node> selectedNodes = new ArrayList<Node>(Arrays.asList(getExplorerManager().getSelectedNodes()));
            LayerObjectNode layerObjectNode = NodeUtils.findNodeForObject(selectedObject);
            if (layerObjectNode != null && selectedNodes.contains(layerObjectNode)) {
                selectedNodes.remove(layerObjectNode);
            }
            getExplorerManager().setSelectedNodes(selectedNodes.toArray(new Node[]{}));
        } catch (PropertyVetoException ex) {
            Exceptions.printStackTrace(ex);
        }
    }

    // ...methods as before, but replace componentActivated and    // componentDeactivated with e.g.:    public void addNotify() {
    // place following methods in jpanel using this explorer manager
//    public void addNotify(){
//        super.addNotify();
//        ExplorerUtils.activateActions(manager, true);
//    }
//    public void removeNotify() {
//        ExplorerUtils.activateActions(manager, false);
//        super.removeNotify();
//    }
    private class EventProducer extends BaseGuiEventProducer {

        public EventProducer() {
            super(LayerEventKey.class);
        }
    }

    private class ExplorerChangeListener implements PropertyChangeListener {

        @Override
        public void propertyChange(PropertyChangeEvent evt) {
            // handle change of node selection

            Map<Layer, Set<LayerObject>> selectedObjectMap = new HashMap<Layer, Set<LayerObject>>();

            if (ExplorerManager.PROP_SELECTED_NODES.equals(evt.getPropertyName())) {
                Node[] nodes = (Node[]) evt.getNewValue();
                for (Node node : nodes) {
                    if (node instanceof LayerObjectNode) {
                        LayerObjectNode objectNode = (LayerObjectNode) node;
                        Node parentNode = objectNode.getParentNode();

                        if (parentNode instanceof LayerNode) {
                            Layer layer = ((LayerNode) parentNode).getLayer();

                            if (!selectedObjectMap.containsKey(layer) || selectedObjectMap.get(layer) == null) {
                                Set<LayerObject> objects = new HashSet<LayerObject>();
                                objects.add(objectNode.getLayerObject());
                                selectedObjectMap.put(layer, objects);
                            } else {
                                selectedObjectMap.get(layer).add(objectNode.getLayerObject());
                            }
                        }
                    }
                }
                for (Layer layer : selectedObjectMap.keySet()) {
                    LayerObjectSelection los = new LayerObjectSelection(selectedObjectMap.get(layer), layer);
                    NewObjectSelectionEvent event = new NewObjectSelectionEvent(los);
                    eventProducer.triggerEvent(event);
                }
            }
        }
    }

    private class NewLayerListener implements GuiEventListener<Layer> {

        @Override
        public void onEvent(GuiEvent<Layer> event) {
            addLayer(event.getContent());
        }

        @Override
        public boolean listensTo(GuiEventKey guiEventKey) {
            return LayerEventKey.NEW_LAYER.equals(guiEventKey);
        }

        @Override
        public String getModuleName() {
            return "geoviewer-jxmap";
        }
    }
}
